-------------------------------------------------------------------------------
-- Locals                                                                    --
-------------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("ClosetGnomeMount")
local lastSetBeforeMounting = nil

local tourist = AceLibrary:HasInstance("Tourist-2.0") and AceLibrary("Tourist-2.0") or nil
local inBattleground = nil

-------------------------------------------------------------------------------
-- Localization                                                              --
-------------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Mount"] = true,
	["Dismount"] = true,

	["Battleground"] = true,
	["Toggle whether the addon is enabled in battlegrounds or not."] = true,
	["What set to equip when you dismount."] = true,
	["What set to equip when you mount."] = true,
	["<set>"] = true,

	["Could not find a set named %q. Please add one."] = true,
} end)

L:RegisterTranslations("zhCN", function() return {
	["Mount"] = "骑乘换装",
	["Could not find a set named %q. Please add one."] = "无法找到名为 qs 的设置. 请添加一个.",
} end)

L:RegisterTranslations("koKR", function() return {
	["Mount"] = "탈것",
	["Could not find a set named %q. Please add one."] = "세트 이름 %q|1을;를 찾을수 없습니다. 하나를 추가하세요.",
} end)

L:RegisterTranslations("esES", function() return {
	["Mount"] = "Montar",
	["Dismount"] = "Desmontar",

	["What set to equip when you dismount."] = "Qu\195\169 conjunto equipar cuando desmontas.",
	["What set to equip when you mount."] = "Qu\195\169 conjunto equipar cuando montas.",
	["<set>"] = "<conjunto>",

	["Could not find a set named %q. Please add one."] = "No se ha podido encontrar el conjunto llamado %q. Por favor, a\195\177ade uno.",
} end)

-------------------------------------------------------------------------------
-- Addon declaration                                                         --
-------------------------------------------------------------------------------

ClosetGnomeMount = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceEvent-2.0", "AceHook-2.1")
ClosetGnomeMount:RegisterDB("ClosetGnomeMountDB")
ClosetGnomeMount:RegisterDefaults("char", {
	dismount = nil,
	mount = nil,
	enabledInBG = true,
})

ClosetGnomeMount:RegisterChatCommand({"/cgmount"}, {
	type = "group",
	name = ClosetGnomeMount.name,
	desc = ClosetGnomeMount.notes,
	args = {
		[L["Dismount"]] = {
			type = "text",
			name = L["Dismount"],
			desc = L["What set to equip when you dismount."],
			get = function() return ClosetGnomeMount.db.char.dismount end,
			set = function(v) ClosetGnomeMount.db.char.dismount = v end,
			validate = function(input) return type(input) == "string" or input == nil end,
			usage = L["<set>"],
		},
		[L["Mount"]] = {
			type = "text",
			name = L["Mount"],
			desc = L["What set to equip when you mount."],
			get = function() return ClosetGnomeMount.db.char.mount end,
			set = function(v) ClosetGnomeMount.db.char.mount = v end,
			validate = function(input) return type(input) == "string" or input == nil end,
			usage = L["<set>"],
		},
		[L["Battleground"]] = {
			type = "toggle",
			name = L["Battleground"],
			desc = L["Toggle whether the addon is enabled in battlegrounds or not."],
			get = function() return ClosetGnomeMount.db.char.enabledInBG end,
			set = function(v)
				ClosetGnomeMount:ToggleBattleground(v)
			end,
			hidden = function() return not tourist end,
		},
	},
})

-------------------------------------------------------------------------------
-- Initialization                                                            --
-------------------------------------------------------------------------------

function ClosetGnomeMount:OnEnable()
	if not ClosetGnome then return end

	self:RegisterEvent("SpecialEvents_Mounted")
	self:RegisterEvent("SpecialEvents_Dismounted")
	self:RegisterEvent("ClosetGnome_WearSet")
	self:Hook(ClosetGnome, "HideSetFromUI")

	if tourist and not self.db.char.enabledInBG then
		self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
		self:ZONE_CHANGED_NEW_AREA()
	end

	if not ClosetGnome:HasSet(L["Mount"]) then
		self:Print(string.format(L["Could not find a set named %q. Please add one."], L["Mount"]))
	else
		-- Force a menu update to hide the mount set properly.
		ClosetGnome:UpdateSetMenu()
	end
end

-------------------------------------------------------------------------------
-- Util                                                                      --
-------------------------------------------------------------------------------

function ClosetGnomeMount:GetMountSet()
	return self.db.char.mount or L["Mount"]
end

function ClosetGnomeMount:GetDismountSet()
	return self.db.char.dismount or L["Dismount"]
end

function ClosetGnomeMount:ToggleBattleground(enabledInBG)
	if tourist then
		self.db.char.enabledInBG = enabledInBG and true or false

		if not enabledInBG then
			if self:IsEventRegistered("ZONE_CHANGED_NEW_AREA") then
				self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
				inBattleground = nil
			end
		else
			if not self:IsEventRegistered("ZONE_CHANGED_NEW_AREA") then
				self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
				self:ZONE_CHANGED_NEW_AREA()
			end
		end
	end
end

-------------------------------------------------------------------------------
-- Events                                                                    --
-------------------------------------------------------------------------------

function ClosetGnomeMount:ZONE_CHANGED_NEW_AREA()
	if not tourist then return end

	inBattleground = tourist:IsBattleground(GetZoneText())
end

function ClosetGnomeMount:HideSetFromUI(_, set)
	if set and set == L["Mount"] or set == L["Dismount"] then
		return true
	end
	return self.hooks[ClosetGnome]["HideSetFromUI"](_, set)
end

function ClosetGnomeMount:ClosetGnome_WearSet(name)
	if name == self:GetMountSet() or name == self:GetDismountSet() then return end
	lastSetBeforeMounting = name
end

function ClosetGnomeMount:SpecialEvents_Mounted()
	if inBattleground and not self.db.profile.enabledInBG then return end

	local set = self:GetMountSet()
	if not ClosetGnome:HasSet(set) then
		self:Print(string.format(L["Could not find a set named %q. Please add one."], set))
		return
	end
	ClosetGnome:WearSet(set)
end

function ClosetGnomeMount:SpecialEvents_Dismounted()
	if inBattleground and not self.db.profile.enabledInBG then return end

	local set = self:GetDismountSet()
	if ClosetGnome:HasSet(set) then
		ClosetGnome:WearSet(set)
	elseif ClosetGnome:HasSet(lastSetBeforeMounting) then
		ClosetGnome:WearSet(lastSetBeforeMounting)
	end
end

